@@ -62,6 +62,17 @@ pub trait AcceptorExt: Acceptor {
     {
         Box::new(WrappedAcceptor(self))
     }
+
+    /// Consume this acceptor and return a new TLS acceptor.
+    #[cfg(feature = "tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
+    #[must_use]
+    fn tls(self, config: TlsConfig) -> IoResult<TlsAcceptor<Self>>
+    where
+        Self: Sized,
+    {
+        TlsAcceptor::new(self, config)
+    }
 }
 
 impl<T: Acceptor> AcceptorExt for T {}
@@ -102,7 +113,7 @@ pub trait Listener: Send {
     where
         Self: Sized,
     {
-        TlsListener::new(config, self)
+        TlsListener::new(self, config)
     }
 }
 
